
/**
  * `db-image`
  *
  *   Lazy fetches and loads an image from firebase cloud firestore.
  *
  * @customElement
  * @polymer
  * @demo demo/index.html
  *
  *
  **/


import {
  AppElement, 
  html
}                 from '@longlost/app-element/app-element.js';
import {
  isOnScreen, 
  schedule,
  wait
}                 from '@longlost/utils/utils.js';
import htmlString from './db-image.html';
import services   from '@longlost/services/services.js';
import '@polymer/iron-image/iron-image.js';


class DbImage extends AppElement {
  static get is() { return 'db-image'; }

  static get template() {
    return html([htmlString]);
  }


  static get properties() {
    return {
      // Image alt tag.
      alt: {
        type: String,
        value: ''
      },
      // Firestore collection name.
      coll: String,
      // Firestore document name.
      doc: String,
      // Firestore document prop.
      field: {
        type: String,
        value: 'images'
      },
      // Use 'index' to access db image by index.
      // If using index, do not set 'name'.
      index: {
        type: Number,
        value: 0
      },
      // Use 'name' to access db image by name.
      // If using name, do not set 'index'.
      name: String,
      // Set to true to use the fullsize 
      // original image file, instead of
      // the optimized version.
      original: Boolean,
      // Image sizing type.
      sizing: {
        type: String,
        value: 'cover' // Or 'contain'.
      },
      // The distance in pixels to pad
      // to the carousel trigger threshold.
      // For instance, 0 would mean that the
      // next lazy image would not start to download
      // until a single pixel intersects the edge of
      // the carousel.
      // Or 128 means that the image would start to
      // download 128px before the next image comes
      // into view.
      trigger: {
        type: Number,
        value: 0
      },
      // Cached image data object from Firestore,
      // at a particular field. coll -> doc -> field
      _data: {
        type: Object,
        computed: '__computeData(_rawData, field)'
      },
      // Set after _src is computed by
      // awaiting a schedule call.
      // 'iron-image' src.
      _lazySrc: String,
      // 'iron-image' placeholder.
      // Is cleared after <iron-image>
      // src is loaded. 
      _placeholder: String,
      // Raw Firestore data. coll -> doc
      _rawData: Object,
      // Temp val that sets '_lazySrc'.
      _src: {
        type: String,
        computed: '__computeSrc(_data, index, name, original)'
      },

      _thumbnail: {
        type: String,
        computed: '__computeThumbnail(_data, index, name)'
      }
 
    }
  }


  static get observers() {
    return [
      '__missingAlt(alt)',
      '__collDocFieldChanged(coll, doc, trigger)',
      '__srcChanged(_src)',
      '__thumbnailChanged(_thumbnail)'
    ];
  }


  __computeData(rawData, field) {
    if (!rawData || !field) { return; }
    return rawData[field];
  }


  __computeSrc(data, index, name, useOriginal) {
    if (!data) { return '#'; }

    if (name) {
      const {optimized, original} = data[name];
      return useOriginal ? original : optimized;
    }

    if (typeof index === 'number') {
      const {optimized, original} = 
        Object.values(data).find(value => 
          value.index === index);
      return useOriginal ? original : optimized;
    }
  }


  __computeThumbnail(data, index, name) {
    if (!data) { return '#'; }

    if (name) {
      const {thumbnail} = data[name];
      return thumbnail;
    }

    if (typeof index === 'number') {
      const {thumbnail} = 
        Object.values(data).find(value => 
          value.index === index);
      return thumbnail;
    }
  }


  __missingAlt(alt) {
    if (!alt) {
      console.warn(this, ' Missing alt tag.', coll, '/', doc, '/', field);
    } 
  }


  async __collDocFieldChanged(coll, doc, trigger) {
    try {      
      if (!coll || !doc || typeof trigger !== 'number') { return; }

      await schedule(); // Allow first layout/paint before measuring.
      await isOnScreen(this, trigger);

      this._rawData = await services.get({
        coll: this.coll, 
        doc:  this.doc
      });
    }
    catch (error) {
      if (error === 'Element removed.') { return; }
      console.error(error);
    }
  }


  async __srcChanged(src) {
    if (!src) { return '#'; }

    await schedule();
    this._lazySrc = src;
  }


  __thumbnailChanged(thumbnail) {
    if (!thumbnail) { return '#'; }

    this._placeholder = thumbnail;
  }

  // Release memory resources.
  async __loadedChanged(event) {
    const {value: loaded} = event.detail;

    if (loaded) {
      await wait(500); // Wait for src to fade in.
      this._lazyPlaceholder = '#';
    }
  }

}

window.customElements.define(DbImage.is, DbImage);
